@@ -732,7 +732,7 @@ class cftimeTestCase(unittest.TestCase):
         # issue #140 (fractional seconds in reference date)
         d = datetime.strptime('2018-01-23 09:27:10.950000',"%Y-%m-%d %H:%M:%S.%f")
         units = 'seconds since 2018-01-23 09:31:42.94'
-        assert(cftime.date2num(d, units) == -271.99)
+        assert(float(cftime.date2num(d, units)) == -271.99)
         # issue 143 - same answer for arrays vs scalars.
         units = 'seconds since 1970-01-01 00:00:00'
         times_in = [1261440000.0, 1261440001.0, 1261440002.0, 1261440003.0,
@@ -751,7 +751,7 @@ class cftimeTestCase(unittest.TestCase):
         # issue #152 add isoformat()
         assert(d.isoformat()[0:24] == '2009-12-22T00:00:00.0156')
         assert(d.isoformat(sep=' ')[0:24] == '2009-12-22 00:00:00.0156')
-        assert(d.isoformat(sep=' ',timespec='milliseconds') == '2009-12-22 00:00:00.015')
+        assert(d.isoformat(sep=' ',timespec='milliseconds') == '2009-12-22 00:00:00.016')
         assert(d.isoformat(sep=' ',timespec='seconds') == '2009-12-22 00:00:00')
         assert(d.isoformat(sep=' ',timespec='minutes') == '2009-12-22 00:00')
         assert(d.isoformat(sep=' ',timespec='hours') == '2009-12-22 00')
@@ -921,6 +921,9 @@ class cftimeTestCase(unittest.TestCase):
         assert(not cftime.is_leap_year(1,calendar='standard',has_year_zero=True))
         assert(not cftime.is_leap_year(1,calendar='365_day'))
         assert(cftime.is_leap_year(1,calendar='366_day'))
+        # num2date should not fail on an empty int array (issue #287)
+        d = cftime.num2date(np.array([], dtype="int64"), "days since 1970-01-01",\
+            calendar="proleptic_gregorian", only_use_cftime_datetimes=True)
 
 
 class TestDate2index(unittest.TestCase):
@@ -1686,6 +1689,59 @@ def test_string_format():
     assert dt.strftime('%H%m%d') == '{0:%H%m%d}'.format(dt)
     assert 'the year is 2000' == 'the year is {dt:%Y}'.format(dt=dt)
 
+
+def test_string_format2():
+    dt = cftime.datetime(-4713, 1, 1, 12, 0, 0, 10)
+    # check a given format string acts like strftime
+    assert dt.strftime('%H%m%d') == '{0:%H%m%d}'.format(dt)
+    assert dt.strftime() == '-4713-01-01 12:00:00'
+    assert dt.strftime('%Y-%m-%d %H:%M:%S') == '-4713-01-01 12:00:00'
+    assert dt.strftime('%Y-%m-%d %H:%M:%S.%f') == '-4713-01-01 12:00:00.000010'
+    assert dt.strftime('%d.%m.%Y %H:%M:%S.%f') == '01.01.-4713 12:00:00.000010'
+    dt = cftime.datetime(-713, 1, 1, 12, 0, 0, 10)
+    assert dt.strftime('%H%m%d') == '{0:%H%m%d}'.format(dt)
+    assert dt.strftime() == '-0713-01-01 12:00:00'
+    assert dt.strftime('%Y-%m-%d %H:%M:%S') == '-0713-01-01 12:00:00'
+    assert dt.strftime('%Y-%m-%d %H:%M:%S.%f') == '-0713-01-01 12:00:00.000010'
+    assert dt.strftime('%d.%m.%Y %H:%M:%S.%f') == '01.01.-0713 12:00:00.000010'
+
+def test_strptime():
+    d = cftime.datetime.strptime('24/Aug/2004:17:57:26 +0200', '%d/%b/%Y:%H:%M:%S %z',calendar='julian',has_year_zero=True)
+    assert(repr(d) == "cftime.datetime(2004, 8, 24, 15, 57, 26, 0, calendar='julian', has_year_zero=True)")
+    d = cftime.datetime.strptime("0000-02-30",\
+             "%Y-%m-%d",calendar='360_day',has_year_zero=True)
+    assert(repr(d) == "cftime.datetime(0, 2, 30, 0, 0, 0, 0, calendar='360_day', has_year_zero=True)")
+    d = cftime.datetime.strptime('-99999-02-29 10:18:32.926',\
+             '%Y-%m-%d %H:%M:%S.%f',calendar='366_day')
+    assert(repr(d) == "cftime.datetime(-99999, 2, 29, 10, 18, 32, 926000, calendar='all_leap', has_year_zero=True)")
+    d = cftime.datetime.strptime('24/Aug/-4712:17:57:26', '%d/%b/%Y:%H:%M:%S',calendar='julian')
+    assert(repr(d) == "cftime.datetime(-4712, 8, 24, 17, 57, 26, 0, calendar='julian', has_year_zero=False)")
+    d = cftime.datetime.strptime('24/August/-4712:17:57:26', '%d/%B/%Y:%H:%M:%S',calendar='julian')
+    assert(repr(d) == "cftime.datetime(-4712, 8, 24, 17, 57, 26, 0, calendar='julian', has_year_zero=False)")
+    d = cftime.datetime.strptime("-4712", "%Y", calendar="julian")
+    assert(repr(d) == "cftime.datetime(-4712, 1, 1, 0, 0, 0, 0, calendar='julian', has_year_zero=False)")
+    # should fail with KeyError
+    try:
+        d=cftime.datetime.strptime("2000-45-3", "%G-%V-%u", calendar="noleap")
+    except KeyError:
+        pass
+    else:
+        raise AssertionError
+
+
+def test_string_isoformat():
+    dt = cftime.datetime(-4713, 1, 1, 12, 0, 0, 10)
+    assert dt.isoformat() == '-4713-01-01T12:00:00.000010'
+    assert dt.isoformat(' ', 'days') == '-4713-01-01'
+    assert dt.isoformat(' ', 'seconds') == '-4713-01-01 12:00:00'
+    assert dt.isoformat(' ', 'microseconds') == '-4713-01-01 12:00:00.000010'
+    dt = cftime.datetime(-713, 1, 1, 12, 0, 0, 10)
+    assert dt.isoformat() == '-0713-01-01T12:00:00.000010'
+    assert dt.isoformat(' ', 'days') == '-0713-01-01'
+    assert dt.isoformat(' ', 'seconds') == '-0713-01-01 12:00:00'
+    assert dt.isoformat(' ', 'microseconds') == '-0713-01-01 12:00:00.000010'
+
+
 def test_dayofyr_after_replace(date_type):
     date = date_type(1, 1, 1)
     assert date.dayofyr == 1
@@ -2048,11 +2104,16 @@ def test_date2num_num2date_roundtrip(encoding_units, freq, calendar):
         assert encoded.dtype == np.int64
         np.testing.assert_equal(decoded, times)
     else:
+        # if sys.platform.startswith("win"):
+        #     assert encoded.dtype == np.float64
+        # else:
+        #     assert encoded.dtype == np.float128
         assert encoded.dtype == np.float64
         tolerance = timedelta(microseconds=2000)
         meets_tolerance = np.abs(decoded - times) <= tolerance
         assert np.all(meets_tolerance)
 
+
 def test_date2num_missing_data():
     # Masked array
     a = [
@@ -2064,7 +2125,7 @@ def test_date2num_missing_data():
     mask = [True, False, True, False]
     array = np.ma.array(a, mask=mask)
     out = date2num(array, units="days since 2000-12-01", calendar="standard")
-    assert ((out == np.ma.array([-99, 1, -99, 3] , mask=mask)).all())
+    assert ((out == np.ma.array([-99, 1, -99, 3], mask=mask)).all())
     assert ((out.mask == mask).all())
 
     # Scalar masked array
@@ -2101,5 +2162,58 @@ def test_num2date_empty_array():
     np.testing.assert_equal(result, expected)
 
 
+DATEPARSE_ERROR_TESTS = [
+    ("foo", "In general, units must be"),
+    ("months", "'months since' units only allowed"),
+    ("common_years", "'common_years' units only allowed")
+]
+
+
+@pytest.mark.parametrize(("units", "match"), DATEPARSE_ERROR_TESTS)
+def test_num2date_unrecognized_units(units, match):
+    with pytest.raises(ValueError, match=match):
+        num2date(0.0, units=f"{units} since 2000-01-01", calendar="standard")
+
+
+@pytest.mark.parametrize(("units", "match"), DATEPARSE_ERROR_TESTS)
+def test_date2num_unrecognized_units(units, match):
+    date = cftime.datetime(2000, 1, 1, calendar="standard")
+    with pytest.raises(ValueError, match=match):
+        date2num(date, units=f"{units} since 2000-01-01", calendar="standard")
+
+
+def test_num2date_precision():
+    if sys.platform.startswith("win"):
+        pytest.skip("skipping tests that require float128 on windows")
+    testdates = [(1271, 3, 18, 19, 41, 33),
+                 (1271, 3, 18, 19, 41, 32, 999998)]
+    unitinc = ['microseconds', 'seconds', 'minutes', 'hours', 'days']
+    for cc in ['standard', 'gregorian', 'julian', 'proleptic_gregorian',
+               'noleap', 'all_leap', '365_day', '366_day', '360_day']:
+        for uinc in unitinc:
+            if cc in ['standard', 'gregorian', 'julian']:
+                units = uinc + ' since -4713-01-01 12:00:00'
+            elif cc in ['proleptic_gregorian']:
+                units = uinc + ' since -4714-01-01 12:00:00'
+            elif cc in ['noleap', 'all_leap', '365_day', '366_day', '360_day']:
+                units = uinc + ' since 0000-01-01 12:00:00'
+            # scalar
+            date = datetimex(*testdates[0], calendar=cc)
+            num = date2num(date, units, calendar=cc, longdouble=True)
+            date2 = num2date(num, units, calendar=cc)
+            assert date == date2
+            # array
+            date = [ datetimex(*dd, calendar=cc) for dd in testdates ]
+            num = date2num(date, units, calendar=cc, longdouble=True)
+            date2 = num2date(num, units, calendar=cc)
+            for i in range(len(date)):
+                assert date[i] == date2[i]
+            # masked array
+            num = np.ma.array(num, mask=(True, False))
+            date2 = num2date(num, units, calendar=cc)
+            assert np.ma.is_masked(date2[0])
+            assert date[1] == date2[1]
+
+
 if __name__ == '__main__':
     unittest.main()
